/*******************************************************************************
 * Copyright (c) 2017 Exyte
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Yuri Strot - initial API and Implementation
 ******************************************************************************/
package org.eclipse.ui.glance.controls.tree;

import org.eclipse.core.runtime.ListenerList;

import org.eclipse.ui.glance.sources.ITextBlock;
import org.eclipse.ui.glance.sources.ITextBlockListener;
import org.eclipse.ui.glance.sources.TextChangedEvent;

public class TreeItemContent implements ITextBlock {

	private TreeNode node;
	private String text;
	private ListenerList listeners = new ListenerList();
	private int column;

	public TreeItemContent(TreeNode node, String text, int column) {
		this.node = node;
		node.items.add(this);
		this.text = text;
		this.column = column;
	}

	public TreeNode getNode() {
		return node;
	}

	@Override
	public String getText() {
		return text;
	}

	public void setText(String text) {
		if (text.equals(this.text))
			return;
		int length = text.length();
		TextChangedEvent event = new TextChangedEvent(0, length, this.text);
		this.text = text;
		for (Object object : listeners.getListeners()) {
			ITextBlockListener listener = (ITextBlockListener) object;
			listener.textChanged(event);
		}
	}

	@Override
	public void addTextBlockListener(ITextBlockListener listener) {
		listeners.add(listener);
	}

	@Override
	public void removeTextBlockListener(ITextBlockListener listener) {
		listeners.remove(listener);
	}

	@Override
	public int compareTo(ITextBlock that) {
		TreeItemContent item = (TreeItemContent) that;
		int diff = this.node.compareTo(item.node);
		if (diff != 0) {
			return diff;
		}
		return column - item.column;
	}

	@Override
	public String toString() {
		StringBuilder buffer = new StringBuilder();
		buffer.append("(");
		buffer.append(text);
		buffer.append(", ");
		buffer.append(column);
		buffer.append(")");
		return buffer.toString();
	}

}
